from partd.file import File

import shutil
import os


def test_partd():
    with File() as p:
        p.append({'x': b'Hello', 'y': b'abc'})
        p.append({'x': b'World!', 'y': b'def'})
        assert os.path.exists(p.filename('x'))
        assert os.path.exists(p.filename('y'))

        result = p.get(['y', 'x'])
        assert result == [b'abcdef', b'HelloWorld!']

        assert p.get('z') == b''

        with p.lock:  # uh oh, possible deadlock
            result = p.get(['x'], lock=False)

    assert not os.path.exists(p.path)


def test_key_tuple():
    with File() as p:
        p.append({('a', 'b'): b'123'})
        assert os.path.exists(p.filename(('a', 'b')))


def test_iset():
    with File() as p:
        p.iset('x', b'123')
        assert 'x' in p._iset_seen
        assert 'y' not in p._iset_seen
        p.iset('x', b'123')
        p.iset('x', b'123')
        assert p.get('x') == b'123'


def test_nested_get():
    with File() as p:
        p.append({'x': b'1', 'y': b'2', 'z': b'3'})
        assert p.get(['x', ['y', 'z']]) == [b'1', [b'2', b'3']]


def test_drop():
    with File() as p:
        p.append({'x': b'123'})
        p.iset('y', b'abc')
        assert p.get('x') == b'123'
        assert p.get('y') == b'abc'

        p.drop()
        assert p.get('x') == b''
        assert p.get('y') == b''

        p.append({'x': b'123'})
        p.iset('y', b'def')
        assert p.get('x') == b'123'
        assert p.get('y') == b'def'


def test_del():
    f = File()

    assert f.path
    assert os.path.exists(f.path)

    f.__del__()
    assert not os.path.exists(f.path)

    with File('Foo') as p:
        p.__del__()
        assert os.path.exists(p.path)


def test_specify_dirname():
    with File(dir=os.getcwd()) as f:
        assert os.getcwd() in f.path



from partd.utils import frame, framesplit
import struct


def test_frame():
    assert frame(b'Hello') == struct.pack('Q', 5) + b'Hello'


def test_framesplit():
    L = [b'Hello', b'World!', b'123']
    assert list(framesplit(b''.join(map(frame, L)))) == L



from __future__ import absolute_import

import pytest
np = pytest.importorskip('numpy')  # noqa

import pickle

import partd
from partd.numpy import Numpy


def test_numpy():
    dt = np.dtype([('a', 'i4'), ('b', 'i2'), ('c', 'f8')])
    with Numpy() as p:
        p.append({'a': np.array([10, 20, 30], dtype=dt['a']),
                  'b': np.array([ 1,  2,  3], dtype=dt['b']),
                  'c': np.array([.1, .2, .3], dtype=dt['c'])})
        p.append({'a': np.array([70, 80, 90], dtype=dt['a']),
                  'b': np.array([ 7,  8,  9], dtype=dt['b']),
                  'c': np.array([.7, .8, .9], dtype=dt['c'])})

        result = p.get(['a', 'c'])
        assert (result[0] == np.array([10, 20, 30, 70, 80, 90],dtype=dt['a'])).all()
        assert (result[1] == np.array([.1, .2, .3, .7, .8, .9],dtype=dt['c'])).all()

        with p.lock:  # uh oh, possible deadlock
            result = p.get(['a'], lock=False)


def test_nested():
    with Numpy() as p:
        p.append({'x': np.array([1, 2, 3]),
                 ('y', 1): np.array([4, 5, 6]),
                 ('z', 'a', 3): np.array([.1, .2, .3])})
        assert (p.get(('z', 'a', 3)) == np.array([.1, .2, .3])).all()


def test_serialization():
    with Numpy() as p:
        p.append({'x': np.array([1, 2, 3])})
        q = pickle.loads(pickle.dumps(p))
        assert (q.get('x') == [1, 2, 3]).all()


array_of_lists = np.empty(3, dtype='O')
array_of_lists[:] = [[1, 2], [3, 4], [5, 6]]


@pytest.mark.parametrize('x', [np.array(['Alice', 'Bob', 'Charlie'], dtype='O'),
                               array_of_lists])
def test_object_dtype(x):
    with Numpy() as p:
        p.append({'x': x})
        p.append({'x': x})
        assert isinstance(p.get('x'), np.ndarray)
        assert (p.get('x') == np.concatenate([x, x])).all()


def test_datetime_types():
    x = np.array(['2014-01-01T12:00:00'], dtype='M8[us]')
    y = np.array(['2014-01-01T12:00:00'], dtype='M8[s]')
    with Numpy() as p:
        p.append({'x': x, 'y': y})
        assert p.get('x').dtype == x.dtype
        assert p.get('y').dtype == y.dtype


def test_non_utf8_bytes():
    a = np.array([b'\xc3\x28', b'\xa0\xa1', b'\xe2\x28\xa1', b'\xe2\x82\x28',
                  b'\xf0\x28\x8c\xbc'], dtype='O')
    s = partd.numpy.serialize(a)
    assert (partd.numpy.deserialize(s, 'O') == a).all()



from partd.file import File
from partd.encode import Encode

import zlib
import shutil
import os


def test_partd():
    with Encode(zlib.compress, zlib.decompress, b''.join) as p:
        p.append({'x': b'Hello', 'y': b'abc'})
        p.append({'x': b'World!', 'y': b'def'})

        result = p.get(['y', 'x'])
        assert result == [b'abcdef', b'HelloWorld!']

        assert p.get('z') == b''

        with p.lock:  # uh oh, possible deadlock
            result = p.get(['x'], lock=False)


def test_ensure():
    with Encode(zlib.compress, zlib.decompress, b''.join) as p:
        p.iset('x', b'123')
        p.iset('x', b'123')
        p.iset('x', b'123')
        assert p.get('x') == b'123'



from partd import File
from partd.core import token, escape_filename, filename
from partd import core
import os
import shutil
from contextlib import contextmanager


def test_partd():
    path = 'tmp.partd'

    with File(path) as p:
        p.append({'x': b'Hello', 'y': b'abc'})
        p.append({'x': b'World!', 'y': b'def'})
        assert os.path.exists(p.filename('x'))
        assert os.path.exists(p.filename('y'))

        result = p.get(['y', 'x'])
        assert result == [b'abcdef', b'HelloWorld!']

        assert p.get('z') == b''

        with p.lock:  # uh oh, possible deadlock
            result = p.get(['x'], lock=False)

    assert not os.path.exists(path)


def test_key_tuple():
    with File('foo') as p:
        p.append({('a', 'b'): b'123'})
        assert os.path.exists(os.path.join(p.path, 'a', 'b'))


def test_ensure():
    with File('foo') as p:
        p.iset('x', b'123')
        p.iset('x', b'123')
        p.iset('x', b'123')

        assert p.get('x') == b'123'


def test_filenames():
    assert token('hello') == 'hello'
    assert token(('hello', 'world')) == os.path.join('hello', 'world')
    assert escape_filename(os.path.join('a', 'b')) == os.path.join('a', 'b')
    assert filename('dir', ('a', 'b')) == os.path.join('dir', 'a', 'b')



import pytest
pytest.importorskip('zmq')

from partd.zmq import Server, keys_to_flush, File, Client
from partd import core, Dict
from threading import Thread
from time import sleep
from contextlib import contextmanager
import pickle

import os
import shutil


def test_server():
    s = Server()
    try:
        s.start()
        s.append({'x': b'abc', 'y': b'1234'})
        s.append({'x': b'def', 'y': b'5678'})

        assert s.get(['x']) == [b'abcdef']
        assert s.get(['x', 'y']) == [b'abcdef', b'12345678']

        assert s.get(['x']) == [b'abcdef']
    finally:
        s.close()


def dont_test_flow_control():
    path = 'bar'
    if os.path.exists('bar'):
        shutil.rmtree('bar')
    s = Server('bar', available_memory=1, n_outstanding_writes=3, start=False)
    p = Client(s.address)
    try:
        listen_thread = Thread(target=s.listen)
        listen_thread.start()
        """ Don't start these threads
        self._write_to_disk_thread = Thread(target=self._write_to_disk)
        self._write_to_disk_thread.start()
        self._free_frozen_sockets_thread = Thread(target=self._free_frozen_sockets)
        self._free_frozen_sockets_thread.start()
        """
        p.append({'x': b'12345'})
        sleep(0.1)
        assert s._out_disk_buffer.qsize() == 1
        p.append({'x': b'12345'})
        p.append({'x': b'12345'})
        sleep(0.1)
        assert s._out_disk_buffer.qsize() == 3

        held_append = Thread(target=p.append, args=({'x': b'123'},))
        held_append.start()

        sleep(0.1)
        assert held_append.isAlive()  # held!

        assert not s._frozen_sockets.empty()

        write_to_disk_thread = Thread(target=s._write_to_disk)
        write_to_disk_thread.start()
        free_frozen_sockets_thread = Thread(target=s._free_frozen_sockets)
        free_frozen_sockets_thread.start()

        sleep(0.2)
        assert not held_append.isAlive()
        assert s._frozen_sockets.empty()
    finally:
        s.close()


@contextmanager
def partd_server(**kwargs):
    with Server(**kwargs) as server:
        with Client(server.address) as p:
            yield (p, server)


def test_partd_object():
    with partd_server() as (p, server):
        p.append({'x': b'Hello', 'y': b'abc'})
        p.append({'x': b'World!', 'y': b'def'})

        result = p.get(['y', 'x'])
        assert result == [b'abcdef', b'HelloWorld!']


def test_delete():
    with partd_server() as (p, server):
        p.append({'x': b'Hello'})
        assert p.get('x') == b'Hello'
        p.delete(['x'])
        assert p.get('x') == b''


def test_iset():
    with partd_server() as (p, server):
        p.iset('x', b'111')
        p.iset('x', b'111')
        assert p.get('x') == b'111'


def test_tuple_keys():
    with partd_server() as (p, server):
        p.append({('x', 'y'): b'123'})
        assert p.get(('x', 'y')) == b'123'


def test_serialization():
    with partd_server() as (p, server):
        p.append({'x': b'123'})
        q = pickle.loads(pickle.dumps(p))
        assert q.get('x') == b'123'


def test_drop():
    with partd_server() as (p, server):
        p.append({'x': b'123'})
        p.drop()
        assert p.get('x') == b''


def dont_test_server_autocreation():
    with Client() as p:
        p.append({'x': b'123'})
        assert p.get('x') == b'123'



from partd.python import dumps, loads


import os
import shutil
from math import sin


def test_pack_unpack():
    data = [1, 2, b'Hello', 'Hello']
    assert loads(dumps(data)) == data

    data = [1, 2, sin]
    assert loads(dumps(data)) == data



from partd.pickle import Pickle


import os
import shutil

def test_pickle():
    with Pickle() as p:
        p.append({'x': ['Hello', 'World!'], 'y': [1, 2, 3]})
        p.append({'x': ['Alice', 'Bob!'], 'y': [4, 5, 6]})
        assert os.path.exists(p.partd.filename('x'))
        assert os.path.exists(p.partd.filename('y'))

        result = p.get(['y', 'x'])
        assert result == [[1, 2, 3, 4, 5, 6],
                          ['Hello', 'World!', 'Alice', 'Bob!']]

        with p.lock:  # uh oh, possible deadlock
            result = p.get(['x'], lock=False)

    assert not os.path.exists(p.partd.path)


def test_ensure():
    with Pickle() as p:
        p.iset('x', [1, 2, 3])
        p.iset('x', [1, 2, 3])

        assert p.get('x') == [1, 2, 3]



from __future__ import absolute_import

import pytest
pytest.importorskip('pandas')  # noqa

import numpy as np
import pandas as pd
import pandas.util.testing as tm
import os

from partd.pandas import PandasColumns, PandasBlocks, serialize, deserialize


df1 = pd.DataFrame({'a': [1, 2, 3],
                    'b': [1., 2., 3.],
                    'c': ['x', 'y', 'x']}, columns=['a', 'b', 'c'],
                    index=pd.Index([1, 2, 3], name='myindex'))

df2 = pd.DataFrame({'a': [10, 20, 30],
                    'b': [10., 20., 30.],
                    'c': ['X', 'Y', 'X']}, columns=['a', 'b', 'c'],
                    index=pd.Index([10, 20, 30], name='myindex'))


def test_PandasColumns():
    with PandasColumns() as p:
        assert os.path.exists(p.partd.partd.path)

        p.append({'x': df1, 'y': df2})
        p.append({'x': df2, 'y': df1})
        assert os.path.exists(p.partd.partd.filename('x'))
        assert os.path.exists(p.partd.partd.filename(('x', 'a')))
        assert os.path.exists(p.partd.partd.filename(('x', '.index')))
        assert os.path.exists(p.partd.partd.filename('y'))

        result = p.get(['y', 'x'])
        tm.assert_frame_equal(result[0], pd.concat([df2, df1]))
        tm.assert_frame_equal(result[1], pd.concat([df1, df2]))

        with p.lock:  # uh oh, possible deadlock
            result = p.get(['x'], lock=False)

    assert not os.path.exists(p.partd.partd.path)


def test_column_selection():
    with PandasColumns('foo') as p:
        p.append({'x': df1, 'y': df2})
        p.append({'x': df2, 'y': df1})
        result = p.get('x', columns=['c', 'b'])
        tm.assert_frame_equal(result, pd.concat([df1, df2])[['c', 'b']])


def test_PandasBlocks():
    with PandasBlocks() as p:
        assert os.path.exists(p.partd.path)

        p.append({'x': df1, 'y': df2})
        p.append({'x': df2, 'y': df1})
        assert os.path.exists(p.partd.filename('x'))
        assert os.path.exists(p.partd.filename('y'))

        result = p.get(['y', 'x'])
        tm.assert_frame_equal(result[0], pd.concat([df2, df1]))
        tm.assert_frame_equal(result[1], pd.concat([df1, df2]))

        with p.lock:  # uh oh, possible deadlock
            result = p.get(['x'], lock=False)

    assert not os.path.exists(p.partd.path)


@pytest.mark.parametrize('ordered', [False, True])
def test_serialize_categoricals(ordered):
    frame = pd.DataFrame({'x': [1, 2, 3, 4],
                          'y': pd.Categorical(['c', 'a', 'b', 'a'],
                                              ordered=ordered)},
                          index=pd.Categorical(['x', 'y', 'z', 'x'],
                                                ordered=ordered))
    frame.index.name = 'foo'
    frame.columns.name = 'bar'

    for ind, df in [(0, frame), (1, frame.T)]:
        df2 = deserialize(serialize(df))
        tm.assert_frame_equal(df, df2)


def test_serialize_multi_index():
    df = pd.DataFrame({'x': ['a', 'b', 'c', 'a', 'b', 'c'],
                       'y': [1, 2, 3, 4, 5, 6],
                       'z': [7., 8, 9, 10, 11, 12]})
    df = df.groupby([df.x, df.y]).sum()
    df.index.name = 'foo'
    df.columns.name = 'bar'

    df2 = deserialize(serialize(df))
    tm.assert_frame_equal(df, df2)


@pytest.mark.parametrize('base', [
    pd.Timestamp('1987-03-3T01:01:01+0001'),
    pd.Timestamp('1987-03-03 01:01:01-0600', tz='US/Central'),
])
def test_serialize(base):
    df = pd.DataFrame({'x': [
        base + pd.Timedelta(seconds=i)
        for i in np.random.randint(0, 1000, size=10)],
                       'y': list(range(10)),
                       'z': pd.date_range('2017', periods=10)})
    df2 = deserialize(serialize(df))
    tm.assert_frame_equal(df, df2)



from partd.dict import Dict
from partd.file import File
from partd.buffer import Buffer, keys_to_flush
import pickle

import shutil
import os


def test_partd():
    a = Dict()
    b = Dict()
    with Buffer(a, b, available_memory=10) as p:
        p.append({'x': b'Hello', 'y': b'abc'})
        assert a.get(['x', 'y']) == [b'Hello', b'abc']

        p.append({'x': b'World!', 'y': b'def'})
        assert a.get(['x', 'y']) == [b'', b'abcdef']
        assert b.get(['x', 'y']) == [b'HelloWorld!', b'']

        result = p.get(['y', 'x'])
        assert result == [b'abcdef', b'HelloWorld!']

        assert p.get('z') == b''

        with p.lock:  # uh oh, possible deadlock
            result = p.get(['x'], lock=False)


def test_keys_to_flush():
    lengths = {'a': 20, 'b': 10, 'c': 15, 'd': 15, 'e': 10, 'f': 25, 'g': 5}
    assert keys_to_flush(lengths, 0.5) == ['f', 'a']


def test_pickle():
    with Dict() as a:
        with File() as b:
            c = Buffer(a, b)

            c.append({'x': b'123'})

            d = pickle.loads(pickle.dumps(c))

            assert d.get('x') == c.get('x')

            pickled_attrs = ('memory_usage', 'lengths', 'available_memory')
            for attr in pickled_attrs:
                assert hasattr(d, attr)
                assert getattr(d, attr) == getattr(c, attr)
            # special case Dict and File -- some attrs do not pickle
            assert hasattr(d, 'fast')
            assert d.fast.data == c.fast.data
            assert hasattr(d, 'slow')
            assert d.slow.path == c.slow.path



#!/usr/bin/env python

from os.path import exists
from setuptools import setup

setup(name='partd',
      version='1.0.0',
      description='Appendable key-value storage',
      url='http://github.com/dask/partd/',
      maintainer='Matthew Rocklin',
      maintainer_email='mrocklin@gmail.com',
      license='BSD',
      keywords='',
      packages=['partd'],
      install_requires=list(open('requirements.txt').read().strip().split('\n')),
      long_description=(open('README.rst').read() if exists('README.rst')
                        else ''),
      extras_require={'complete': [
          'numpy >= 1.9.0',
          'pandas >=0.19.0',
          'pyzmq',
          'blosc',
      ]},
      zip_safe=False)



from partd.compressed import ZLib


import shutil
import os
import pickle


def test_partd():
    with ZLib() as p:
        p.append({'x': b'Hello', 'y': b'abc'})
        p.append({'x': b'World!', 'y': b'def'})
        assert os.path.exists(p.partd.filename('x'))
        assert os.path.exists(p.partd.filename('y'))

        result = p.get(['y', 'x'])
        assert result == [b'abcdef', b'HelloWorld!']

        assert p.get('z') == b''

        with p.lock:  # uh oh, possible deadlock
            result = p.get(['x'], lock=False)

    assert not os.path.exists(p.partd.path)


def test_pickle():
    with ZLib() as p:
        p.append({'x': b'123'})
        q = pickle.loads(pickle.dumps(p))
        assert q.get('x') == b'123'



from partd.dict import Dict

import shutil
import os


def test_partd():
    with Dict() as p:
        p.append({'x': b'Hello', 'y': b'abc'})
        p.append({'x': b'World!', 'y': b'def'})

        result = p.get(['y', 'x'])
        assert result == [b'abcdef', b'HelloWorld!']

        assert p.get('z') == b''

        with p.lock:  # uh oh, possible deadlock
            result = p.get(['x'], lock=False)


def test_key_tuple():
    with Dict() as p:
        p.append({('a', 'b'): b'123'})
        assert p.get(('a', 'b')) == b'123'


def test_iset():
    with Dict() as p:
        p.iset('x', b'123')
        assert 'x' in p._iset_seen
        assert 'y' not in p._iset_seen
        p.iset('x', b'123')
        p.iset('x', b'123')
        assert p.get('x') == b'123'


def test_delete_non_existent_key():
    with Dict() as p:
        p.append({'x': b'123'})
        p.delete(['x', 'y'])
        assert p.get(['x', 'y']) == [b'', b'']
